@@ -267,6 +267,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
     $user_agent_string = $config_vars{'USCAN_USER_AGENT'}
 	if $config_vars{'USCAN_USER_AGENT'};
     $repack = $config_vars{'USCAN_REPACK'} eq 'yes' ? 1 : 0;
+    $exclusion = $config_vars{'USCAN_EXCLUSION'} eq 'yes' ? 1 : 0;
 }
 
 # Now read the command line arguments
@@ -1465,17 +1466,24 @@ EOF
 	  or uscan_die("unzip binary not found. You need to install the package unzip to be able to repack .zip upstream archives.\n");
 
 	my $newfile_base_gz = "$1.tar.gz";
-	my $tempdir = tempdir ( "uscanXXXX", TMPDIR => 1, CLEANUP => 1 );
-	my $globpattern = "*";
-	my $hidden = ".[!.]*";
+	my $tempdir = tempdir ("uscanXXXX", TMPDIR => 1, CLEANUP => 1);
+	# Parent of the target directory should be under our control
+	$tempdir .= '/repack';
+	mkdir $tempdir or uscan_die("Unable to mkdir($tempdir): $!\n");
 	my $absdestdir = abs_path($destdir);
 	system('unzip', '-q', '-a', '-d', $tempdir, "$destdir/$newfile_base") == 0
-	  or uscan_die("Repacking from zip or jar to tar.gz failed (could not unzip)\n");
-	if (defined glob("$tempdir/$hidden")) {
-	    $globpattern .= " $hidden";
+	    or uscan_die("Repacking from zip or jar to tar.gz failed (could not unzip)\n");
+	my $cwd = cwd();
+	chdir($tempdir) or uscan_die("Unable to chdir($tempdir): $!\n");
+	eval {
+	    spawn(exec => ['tar', '--owner=root', '--group=root', '--mode=a+rX', '-czf', "$absdestdir/$newfile_base_gz", glob('* .[!.]*')],
+		  env => { GZIP => '-n -9' },
+		  wait_child => 1);
+	};
+	if ($@) {
+	    uscan_die("Repacking from zip or jar to tar.gz failed (could not create tarball)\n");
 	}
-	system("cd $tempdir; GZIP='-n -9' tar --owner=root --group=root --mode=a+rX -czf \"$absdestdir/$newfile_base_gz\" $globpattern") == 0
-	  or uscan_die("Repacking from zip or jar to tar.gz failed (could not create tarball)\n");
+	chdir($cwd);
 	unlink "$destdir/$newfile_base";
 	$newfile_base = $newfile_base_gz;
     }
@@ -1484,8 +1492,13 @@ EOF
 			     |tar\.bz2|tbz2?
 			     |tar.lzma|tlz(?:ma?)?
 			     |tar.xz|txz)$/x) {
-	my $filetype = `file -b -k \"$destdir/$newfile_base\"`;
-	unless ($filetype =~ /compressed data/) {
+	my $filetype;
+	eval {
+	    spawn(exec => ['file', '-b', '-k', "$destdir/$newfile_base"],
+		  to_string => \$filetype,
+		  wait_child => 1);
+	};
+	unless (defined $filetype && $filetype =~ /compressed data/) {
 	    uscan_warn "$progname warning: $destdir/$newfile_base does not appear to be a compressed file;\nthe file command says: $filetype\nNot processing this file any further!\n";
 	    return 1;
 	}
@@ -1494,17 +1507,28 @@ EOF
     my $excludesuffix = '+dfsg';
     if ($exclusion) {
 	my $data = Dpkg::Control::Hash->new();
-	$data->load('debian/copyright');
 	my $okformat = qr'http://www.debian.org/doc/packaging-manuals/copyright-format/[.\d]+';
-	if ($data->{'format'} =~ m{^$okformat/?$} and $data->{'files-excluded'} ) {
-	    my $tempdir = tempdir ( "uscanXXXX", TMPDIR => 1, CLEANUP => 1 );
-	    my $globpattern = "*";
-	    my $hidden = ".[!.]*";
-	    if (defined glob("$tempdir/$hidden")) {
-		$globpattern .= " $hidden";
-	    }
+	eval {
+	  $data->load('debian/copyright');
+	  1;
+	} or do {
+	  undef $data;
+	};
+	if (   $data
+	    && defined $data->{'format'}
+	    && $data->{'format'} =~ m{^$okformat/?$}
+	    && $data->{'files-excluded'})
+	{
+	    my $tempdir = tempdir ("uscanXXXX", TMPDIR => 1, CLEANUP => 1);
+	    # Parent of the target directory should be under our control
+	    $tempdir .= '/repack';
+	    mkdir $tempdir or uscan_die("Unable to mkdir($tempdir): $!\n");
 	    my $absdestdir = abs_path($destdir);
-	    unless (system("cd $tempdir; tar -xaf \"$absdestdir/$newfile_base\" 2>/dev/null") == 0) {
+	    eval {
+		spawn(exec => ['tar', '-xa', '-k', '--no-overwrite-dir', '-C', $tempdir, '-f', "$absdestdir/$newfile_base"],
+		      wait_child => 1);
+	    };
+	    if ($@) {
 		print "-- $newfile_base is no tarball.  Try unzip.\n" if $verbose;
 		# try unzip if tar fails - we do want to do something sensible even if no --repack was specified
 		system('command -v unzip >/dev/null 2>&1') >> 8 == 0
@@ -1516,24 +1540,35 @@ EOF
 	    unless ( -d $main_source_dir ) {
 		print STDERR "Error: $main_source_dir is no directory";
 	    }
-	    my $nfiles_before = `find $main_source_dir | wc -l`;
-	    foreach (grep {/\//} split /\s+/, $data->{"files-excluded"}) {
+	    my $file_list;
+	    spawn(exec => ['find', $main_source_dir], wait_child => 1, to_string => \$file_list);
+	    my $nfiles_before = split /\n/, $file_list;
+	    foreach (grep { $_ } split /\s+/, $data->{"files-excluded"}) {
 		# delete trailing '/' because otherwise find -path will fail
-		s?/+$?? ;
+		s?/+$??;
 		# use rm -rf to enable deleting non-empty directories
-		`find $main_source_dir -path "$main_source_dir/$_" | xargs rm -rf`;
-	    };
-	    foreach (grep {/^[^\/]+$/} split /\s+/, $data->{"files-excluded"}) {
-		`find $main_source_dir -type f -name $_ -delete`;
-	    };
-	    my $nfiles_after = `find $main_source_dir | wc -l`;
+		spawn(exec => ['find', $main_source_dir, '-path', "$main_source_dir/$_", '-exec', 'rm', '-rf', '{}', '+'],
+		      wait_child => 1);
+	    }
+	    undef $file_list;
+	    spawn(exec => ['find', $main_source_dir], wait_child => 1, to_string => \$file_list);
+	    my $nfiles_after = split /\n/, $file_list;
 	    if ( $nfiles_before == $nfiles_after ) {
 		print "-- Source tree remains identical - no need for repacking.\n" if $verbose;
 	    } else {
 		my $suffix = 'gz' ;
 		my $newfile_base_dfsg = "${pkg}_${newversion}${excludesuffix}.orig.tar.$suffix" ;
-		system("cd $tempdir; GZIP='-n -9' tar --owner=root --group=root --mode=a+rX -czf \"$absdestdir/$newfile_base_dfsg\" $globpattern") == 0
-		    or die("Excluding files failed (could not create tarball)\n");
+		my $cwd = cwd();
+		chdir($tempdir) or uscan_die("Unable to chdir($tempdir): $!\n");
+		eval {
+		    spawn(exec => ['tar', '--owner=root', '--group=root', '--mode=a+rX', '-czf', "$absdestdir/$newfile_base_dfsg", glob('* .[!.]*')],
+			  env => { GZIP => '-n -9' },
+			  wait_child => 1);
+		};
+		if ($@) {
+		    uscan_die("Excluding files failed (could not create tarball)\n");
+		}
+		chdir($cwd);
 		$symlink = 'files-excluded' # prevent symlinking or renaming
 	    }
 	}
@@ -2171,12 +2206,12 @@ sub get_main_source_dir($$$$) {
     foreach my $file (@files) {
 	unless ($file =~ /^\.\.?/) {
 	    if ( -d "${tempdir}/$file" ) {
-                # HELP: why can't perl move not move directories????
-                system( "mv ${tempdir}/$file $main_source_dir" ) ;
-            } else {
-                move("${tempdir}/$file", $main_source_dir) or die("Unable to move ${tempdir}/$file directory $main_source_dir\n");
-            }
-        }
+		# HELP: why can't perl move not move directories????
+		system('mv', "${tempdir}/$file", $main_source_dir);
+	    } else {
+		move("${tempdir}/$file", $main_source_dir) or die("Unable to move ${tempdir}/$file directory $main_source_dir\n");
+	    }
+	}
     }
     return $main_source_dir;
 }
